
# q-expansion of newform 8036.2.a.c, downloaded from the LMFDB on 26 April 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 8036
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [-1], [3], [0], [3], [4], [0], [7], [6], [6], [10], [2], [1], [-4], [-12], [-6], [-6], [13], [-4], [-9], [-14], [-1], [-12], [12], [-2], [0], [4], [6], [8], [-3], [14], [-18], [-12], [-14], [18], [-4], [-14], [14], [-12], [21], [3], [4], [0], [14], [3], [16], [23], [-8], [-21], [4], [6], [-27], [22], [-12], [6], [12], [15], [-8], [-13], [6], [-26], [-12], [-20], [-12], [-20], [-18], [-28], [23], [-15], [-14], [15], [36], [28], [29], [-16], [27], [15], [-14], [27], [31], [6], [8], [18], [-23], [1], [18], [9], [-16], [-27], [-1], [-30], [-21], [8], [-42], [-4], [-9], [18], [-30], [-38], [14], [20], [-12], [15], [30], [5], [-20], [33], [6], [-30], [34], [28], [-7], [-27], [-44], [20], [12], [-5], [12], [36], [-48], [31], [-22], [15], [12], [1], [15], [-10], [-3], [40], [22], [-16], [6], [-43], [-40], [-18], [-35], [-36], [28], [30], [-12], [-38], [-9], [5], [12], [-38], [3], [49], [3], [-8], [24], [17], [-45], [32], [-3], [-10], [0], [59], [0], [34], [45], [-18], [-6], [23], [0], [-42], [-12], [-43], [28], [38], [6], [51], [22], [-27], [-19], [-5], [30], [53], [-45], [-17], [10], [-34], [-30], [50], [-30], [12], [9], [-58], [-62], [-22], [33], [-50], [51], [-28], [15], [-12], [-54], [14], [-10], [60], [21], [-21], [-8], [-2], [-8], [12], [-12], [16], [-3], [15], [-44], [38], [-6], [-34], [-18], [27], [-17], [-49], [-18], [24], [66], [14], [58], [60], [29], [-6], [-52], [21], [-36], [-53], [0], [59], [-20], [-4], [-45], [19], [3], [-44], [-54], [-30], [24], [-36], [67], [13], [14], [-54], [6], [-2], [-21], [14], [-42], [-37], [-57], [48], [19], [-30], [30], [38], [-68], [-42], [34], [-4], [-3], [40], [52], [-51], [28], [54], [-27], [14], [60], [-32], [62], [-53], [-4], [58], [82], [-60], [-4], [-46], [-51], [48], [-37], [-6], [37], [-56], [3], [-52], [-75], [-8], [-48], [48], [36], [6], [12], [-16], [-72], [55], [-51], [-39], [-17], [31], [36], [-16], [-36], [32], [-70], [-54], [79], [69], [32], [24], [-54], [-57], [32], [-18], [73], [-18], [-84], [-14], [48], [82], [5], [9], [11], [54], [-56], [71], [1], [-45], [27], [62], [-54], [88], [51], [38], [36], [20], [-30], [-56], [-68], [62], [75], [42], [-13], [6], [6], [-35], [-10], [-24], [-30], [-20], [-46], [48], [-26], [86], [-30], [-66], [-36], [6], [90], [-31], [-66], [-39], [-60], [-20], [-16], [72], [-52], [-31], [-63], [-65], [36], [48], [-20], [38], [-12], [-54], [49], [-54], [4], [0], [-63], [-58], [-57], [79], [18], [65], [82], [2], [-81], [98], [0], [69], [-62], [12], [-22], [-47], [51], [8], [0], [31], [-42], [50], [-30], [18], [28], [53], [12], [85], [48], [1], [-3], [-99], [-49], [68], [-42], [66], [43], [-48], [-66], [30], [4], [-48], [51], [-29], [-27], [-3], [-36], [79], [-9]]
